﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PointLib;
using System.Collections;

namespace CollectionTest
{
    public class PointListTest
    {
        // Print Point[]
        void Print(Point[] pArray)
        {
            foreach (Point p in pArray)
            {
                System.Console.WriteLine(p);
            }
        }

        // Print List<Point>
        void Print(List<Point> pList)
        {
            IEnumerator<Point> enumerator = pList.GetEnumerator();
            while (enumerator.MoveNext())
            {
                System.Console.WriteLine(enumerator.Current);
            }
        }
        
        // comparison delegate
        int ComparePointByY(Point p1, Point p2)
        {
            return p1.Y.CompareTo(p2.Y);
        }

        public PointListTest()
        {
            // 1. Point[]
            Point[] pArray1 = new Point[3];
            pArray1[0] = new Point(10, 50);
            pArray1[1] = new Point(-30, 30);
            pArray1[2] = new Point(20, 20);
            System.Console.WriteLine("Point[]!!!");
            Print(pArray1);

            System.Console.WriteLine("After sort Point[]!!!");
            Array.Sort(pArray1);
            Print(pArray1);
            System.Console.WriteLine("After sort Point[] by YComparer!!!");
            Array.Sort(pArray1, new YComparer());
            Print(pArray1);

            // 2. List<Point>
            List<Point> pList1 = new List<Point>();
            pList1.Add(new Point(100, 500));
            pList1.Add(new Point(-300, 300));
            pList1.Add(new Point(200, 200));
            System.Console.WriteLine("List<Point>!!!");

            pList1.Sort(); // Point.CompareTo
            Print(pList1);
            pList1.Sort(new YComparer()); // YComparer
            pList1.Sort(ComparePointByY); // delegate
            pList1.Sort(delegate (Point p, Point q) // anonymous delegate
            {
                return p.X.CompareTo(q.X);
            });
            pList1.Sort((p, q) => p.Y.CompareTo(q.Y)); // lambda

            // == vs equals vs hashCode
            System.Console.WriteLine("== vs equals vs hashCode~~~~~~~~");
            Point p1 = new Point(20, 30);
            Point p2 = new Point(20, 30);
            Point p3 = p1;
            if (p1 == p2) System.Console.WriteLine("p1 == p2"); // if Point override Equals, ==, !=, GetHashCode, then true
            else System.Console.WriteLine("p1 != p2");
            if (p1 == p3) System.Console.WriteLine("p1 == p3");
            else System.Console.WriteLine("p1 != p3");
            if (p1.Equals(p2)) System.Console.WriteLine("p1 equals p2");
            else System.Console.WriteLine("p1 not equals p2");
            if (p1.Equals(p3)) System.Console.WriteLine("p1 equals p3");
            else System.Console.WriteLine("p1 not equals p3");

            // Dictionary does not allow the same key (GetHashCode)
            Dictionary<int, Point> pMap = new Dictionary<int, Point>();
            pMap[1] = p1;
            pMap[2] = p2;
            pMap[3] = p3;
            System.Console.WriteLine("pMap.size=" + pMap.Count()); // 3
            foreach (KeyValuePair<int, Point> e in pMap) {
                System.Console.WriteLine("Key=" + e.Key + " Value=" + e.Value);
            }
            pMap.Remove(2);
            System.Console.WriteLine("after remove Key=2 pMap.size=" + pMap.Count());
            foreach (KeyValuePair<int, Point> e in pMap)
            {
                System.Console.WriteLine("Key=" + e.Key + " Value=" + e.Value);
            }

            // ExceptionError (Dictionary cannot use the same key)
            Dictionary<Point, int> pMap2 = new Dictionary<Point, int>();
            pMap2.Add(p1, 1);
            pMap2.Add(p2, 2); // ExceptionError p1==p2 due to Equals, ==, !=, GetHashCode override
            pMap2.Add(p3, 3);
            System.Console.WriteLine("pMap2.size=" + pMap2.Count());
            foreach (KeyValuePair<Point, int> e in pMap2)
            {
                System.Console.WriteLine("Key=" + e.Key + " Value=" + e.Value);
            }
            pMap.Remove(2);
            System.Console.WriteLine("after remove Key=2 pMap.size=" + pMap.Count());
            foreach (KeyValuePair<Point, int> e in pMap2)
            {
                System.Console.WriteLine("Key=" + e.Key + " Value=" + e.Value);
            }

        }
    }
}
