﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PointLib
{
    public class Point : IComparable<Point>, IEquatable<Point>
    {
        private int x;
        private int y;

        public Point() : this(0, 0) { }

        public Point(int x, int y)
        {
            this.x = x;
            this.y = y;
        }

        public int X
        {
            get { return x; }
            set { x = value; }
        }

        public int Y
        {
            get { return y; }
            set { y = value; }
        }

        public override string ToString()
        {
            return String.Format("({0},{1})", X, Y);
        }

        // object GetHashCode() override
        public override int GetHashCode()
        {
            return x ^ y;
        }

        // object Equals() override
        public override bool Equals(object obj)
        {
            if (!(obj is Point))
                return false;
            return Equals((Point)obj);
        }

        public bool Equals(Point other)
        {
            if (this.x != other.x)
                return false;
            return this.y == other.y;
        }

        // operator==
        public static bool operator ==(Point p1, Point p2)
        {
            return p1.Equals(p2);
        }

        // operator!=
        public static bool operator !=(Point p1, Point p2)
        {
            return !p1.Equals(p2);
        }

        // Comparable<Point> implementation
        public int CompareTo(Point other)
        {
            if (this.X == other.X)
                return this.Y.CompareTo(other.Y);
            return this.X.CompareTo(other.X);
        }
    }

    public class YComparer : IComparer<Point>
    {
        // Comparere<Point> implementation
        public int Compare(Point p1, Point p2)
        {
            if (p1.Y == p2.Y)
                return p1.X.CompareTo(p2.X);
            return p1.Y.CompareTo(p2.Y);
        }
    }
}
